@@ -214,38 +214,6 @@ public:
     GpgME::Error edit(const Key &key, std::unique_ptr<EditInteractor> function, Data &out);
     GpgME::Error startEditing(const Key &key, std::unique_ptr<EditInteractor> function, Data &out);
 
-
-    //
-    // Modern Interface actions. Require 2.1.x
-    //
-    Error startCreateKey (const char *userid,
-                          const char *algo,
-                          unsigned long reserved,
-                          unsigned long expires,
-                          const Key &certkey,
-                          unsigned int flags);
-    Error createKey (const char *userid,
-                     const char *algo,
-                     unsigned long reserved,
-                     unsigned long expires,
-                     const Key &certkey,
-                     unsigned int flags);
-
-    Error addUid(const Key &key, const char *userid);
-    Error startAddUid(const Key &key, const char *userid);
-
-    Error revUid(const Key &key, const char *userid);
-    Error startRevUid(const Key &key, const char *userid);
-
-    Error createSubkey(const Key &key, const char *algo,
-                       unsigned long reserved = 0,
-                       unsigned long expires = 0,
-                       unsigned int flags = 0);
-    Error startCreateSubkey(const Key &key, const char *algo,
-                            unsigned long reserved = 0,
-                            unsigned long expires = 0,
-                            unsigned int flags = 0);
-
     // using TofuInfo::Policy
     Error setTofuPolicy(const Key &k, unsigned int policy);
     Error setTofuPolicyStart(const Key &k, unsigned int policy);
@@ -287,28 +255,14 @@ public:
     //
     // Crypto Operations
     //
-
-    enum DecryptionFlags {
-        // Keep in line with core's flags
-        DecryptNone = 0,
-        DecryptVerify = 1,
-        DecryptUnwrap = 128,
-        DecryptMaxValue = 0x80000000
-    };
+    //
 
     //
     // Decryption
     //
 
-    // Alternative way to set decryption flags as they were added only in
-    // 1.9.0 and so other API can still be used but with 1.9.0 additionally
-    // flags can be set.
-    void setDecryptionFlags (const DecryptionFlags flags);
-
     DecryptionResult decrypt(const Data &cipherText, Data &plainText);
     GpgME::Error startDecryption(const Data &cipherText, Data &plainText);
-    DecryptionResult decrypt(const Data &cipherText, Data &plainText, const DecryptionFlags flags);
-    GpgME::Error startDecryption(const Data &cipherText, Data &plainText, const DecryptionFlags flags);
     DecryptionResult decryptionResult() const;
 
     //
@@ -326,9 +280,7 @@ public:
     //
 
     std::pair<DecryptionResult, VerificationResult> decryptAndVerify(const Data &cipherText, Data &plainText);
-    std::pair<DecryptionResult, VerificationResult> decryptAndVerify(const Data &cipherText, Data &plainText, const DecryptionFlags flags);
     GpgME::Error startCombinedDecryptionAndVerification(const Data &cipherText, Data &plainText);
-    GpgME::Error startCombinedDecryptionAndVerification(const Data &cipherText, Data &plainText, const DecryptionFlags flags);
     // use verificationResult() and decryptionResult() to retrieve the result objects...
 
     //
@@ -352,10 +304,6 @@ public:
     GpgME::Error startSigning(const Data &plainText, Data &signature, SignatureMode mode);
     SigningResult signingResult() const;
 
-    // wrapper for gpgme_set_sender
-    const char *getSender();
-    GpgME::Error setSender(const char *sender);
-
     //
     // Encryption
     //
@@ -367,9 +315,7 @@ public:
         Prepare = 4,
         ExpectSign = 8,
         NoCompress = 16,
-        Symmetric = 32,
-        ThrowKeyIds = 64,
-        EncryptWrap = 128
+        Symmetric = 32
     };
     EncryptionResult encrypt(const std::vector<Key> &recipients, const Data &plainText, Data &cipherText, EncryptionFlags flags);
     GpgME::Error encryptSymmetrically(const Data &plainText, Data &cipherText);
@@ -418,8 +364,8 @@ public:
      * @param file The executable to start.
      * @param argv list of arguments file should be argv[0].
      * @param input The data to be sent through stdin.
-     * @param output The data to be receive the stdout.
-     * @param err The data to receive stderr.
+     * @param output The data to be recieve the stdout.
+     * @param err The data to recieve stderr.
      * @param flags Additional flags.
      *
      * @returns An error or empty error.
