@@ -1,8 +1,6 @@
 /*
   decryptionresult.cpp - wraps a gpgme keygen result
   Copyright (C) 2004 Klarälvdalens Datakonsult AB
-  2016 Bundesamt für Sicherheit in der Informationstechnik
-  Software engineering by Intevation GmbH
 
   This file is part of GPGME++.
 
@@ -22,10 +20,6 @@
   Boston, MA 02110-1301, USA.
 */
 
-#ifdef HAVE_CONFIG_H
- #include "config.h"
-#endif
-
 #include <decryptionresult.h>
 #include "result_p.h"
 #include "util.h"
@@ -51,30 +45,23 @@ public:
         if (res.file_name) {
             res.file_name = strdup(res.file_name);
         }
-        if (res.symkey_algo) {
-            res.symkey_algo = strdup(res.symkey_algo);
-        }
         //FIXME: copying gpgme_recipient_t objects invalidates the keyid member,
         //thus we use _keyid for now (internal API)
         for (gpgme_recipient_t r = res.recipients ; r ; r = r->next) {
             recipients.push_back(*r);
         }
-        res.recipients = nullptr;
+        res.recipients = 0;
     }
     ~Private()
     {
         if (res.unsupported_algorithm) {
             std::free(res.unsupported_algorithm);
         }
-        res.unsupported_algorithm = nullptr;
+        res.unsupported_algorithm = 0;
         if (res.file_name) {
             std::free(res.file_name);
         }
-        res.file_name = nullptr;
-        if (res.symkey_algo) {
-            std::free(res.symkey_algo);
-        }
-        res.symkey_algo = nullptr;
+        res.file_name = 0;
     }
 
     _gpgme_op_decrypt_result res;
@@ -109,7 +96,7 @@ make_standard_stuff(DecryptionResult)
 
 const char *GpgME::DecryptionResult::unsupportedAlgorithm() const
 {
-    return d ? d->res.unsupported_algorithm : nullptr ;
+    return d ? d->res.unsupported_algorithm : 0 ;
 }
 
 bool GpgME::DecryptionResult::isWrongKeyUsage() const
@@ -117,14 +104,9 @@ bool GpgME::DecryptionResult::isWrongKeyUsage() const
     return d && d->res.wrong_key_usage;
 }
 
-bool GpgME::DecryptionResult::isDeVs() const
-{
-    return d && d->res.is_de_vs;
-}
-
 const char *GpgME::DecryptionResult::fileName() const
 {
-    return d ? d->res.file_name : nullptr ;
+    return d ? d->res.file_name : 0 ;
 }
 
 unsigned int GpgME::DecryptionResult::numRecipients() const
@@ -162,21 +144,6 @@ std::vector<GpgME::DecryptionResult::Recipient> GpgME::DecryptionResult::recipie
     return result;
 }
 
-const char *GpgME::DecryptionResult::sessionKey() const
-{
-  return d ? d->res.session_key : nullptr;
-}
-
-const char *GpgME::DecryptionResult::symkeyAlgo() const
-{
-  return d ? d->res.symkey_algo : nullptr;
-}
-
-bool GpgME::DecryptionResult::isLegacyCipherNoMDC() const
-{
-  return d && d->res.legacy_cipher_nomdc;
-}
-
 class GpgME::DecryptionResult::Recipient::Private : public _gpgme_recipient
 {
 public:
@@ -208,7 +175,7 @@ const char *GpgME::DecryptionResult::Recipient::keyID() const
     if (d) {
         return d->_keyid;
     }
-    return nullptr;
+    return 0;
 }
 
 const char *GpgME::DecryptionResult::Recipient::shortKeyID() const
@@ -217,7 +184,7 @@ const char *GpgME::DecryptionResult::Recipient::shortKeyID() const
     if (d) {
         return d->_keyid + 8;
     }
-    return nullptr;
+    return 0;
 }
 
 unsigned int GpgME::DecryptionResult::Recipient::publicKeyAlgorithm() const
@@ -233,7 +200,7 @@ const char *GpgME::DecryptionResult::Recipient::publicKeyAlgorithmAsString() con
     if (d) {
         return gpgme_pubkey_algo_name(d->pubkey_algo);
     }
-    return nullptr;
+    return 0;
 }
 
 GpgME::Error GpgME::DecryptionResult::Recipient::status() const
@@ -252,9 +219,6 @@ std::ostream &GpgME::operator<<(std::ostream &os, const DecryptionResult &result
            << "\n fileName:             " << protect(result.fileName())
            << "\n unsupportedAlgorithm: " << protect(result.unsupportedAlgorithm())
            << "\n isWrongKeyUsage:      " << result.isWrongKeyUsage()
-           << "\n isDeVs                " << result.isDeVs()
-           << "\n legacyCipherNoMDC     " << result.isLegacyCipherNoMDC()
-           << "\n symkeyAlgo:           " << protect(result.symkeyAlgo())
            << "\n recipients:\n";
         const std::vector<DecryptionResult::Recipient> recipients = result.recipients();
         std::copy(recipients.begin(), recipients.end(),
