@@ -1,8 +1,6 @@
 /*
   gpgsignkeyeditinteractor.cpp - Edit Interactor to change the expiry time of an OpenPGP key
   Copyright (C) 2007 Klarälvdalens Datakonsult AB
-  2016 Bundesamt für Sicherheit in der Informationstechnik
-  Software engineering by Intevation GmbH
 
   This file is part of GPGME++.
 
@@ -22,10 +20,6 @@
   Boston, MA 02110-1301, USA.
 */
 
-#ifdef HAVE_CONFIG_H
- #include "config.h"
-#endif
-
 #include "gpgsignkeyeditinteractor.h"
 #include "error.h"
 #include "key.h"
@@ -64,13 +58,6 @@ public:
     std::vector<unsigned int> userIDs;
     std::vector<unsigned int>::const_iterator currentId, nextId;
     unsigned int checkLevel;
-    bool dupeOk;
-    Key key;
-    struct {
-        TrustSignatureTrust trust;
-        std::string depth;
-        std::string scope;
-    } trustSignature;
 
     const char *command() const
     {
@@ -133,9 +120,7 @@ GpgSignKeyEditInteractor::Private::Private()
     userIDs(),
     currentId(),
     nextId(),
-    checkLevel(0),
-    dupeOk(false),
-    trustSignature{TrustSignatureTrust::None, "0", {}}
+    checkLevel(0)
 {
 }
 
@@ -167,10 +152,6 @@ enum SignKeyState {
     SET_TRUST_DEPTH,
     SET_TRUST_REGEXP,
     CONFIRM,
-    CONFIRM2,
-    DUPE_OK,
-    DUPE_OK2,
-    REJECT_SIGN_EXPIRED,
     QUIT,
     SAVE,
     ERROR = EditInteractor::ErrorState
@@ -194,43 +175,28 @@ static GpgSignKeyEditInteractor_Private::TransitionMap makeTable()
 #define addEntry( s1, status, str, s2 ) tab[std::make_tuple( s1, status, str)] = s2
     addEntry(START, GET_LINE, "keyedit.prompt", COMMAND);
     addEntry(COMMAND, GET_BOOL, "keyedit.sign_all.okay", UIDS_ANSWER_SIGN_ALL);
-    addEntry(COMMAND, GET_BOOL, "sign_uid.expired_okay", REJECT_SIGN_EXPIRED);
     addEntry(COMMAND, GET_BOOL, "sign_uid.okay", CONFIRM);
-    addEntry(COMMAND, GET_BOOL, "sign_uid.local_promote_okay", CONFIRM2);
-    addEntry(COMMAND, GET_BOOL, "sign_uid.dupe_okay", DUPE_OK);
-    addEntry(COMMAND, GET_LINE, "trustsig_prompt.trust_value", SET_TRUST_VALUE);
     addEntry(UIDS_ANSWER_SIGN_ALL, GET_BOOL, "sign_uid.okay", CONFIRM);
-    addEntry(UIDS_ANSWER_SIGN_ALL, GET_BOOL, "sign_uid.dupe_okay", DUPE_OK);
     addEntry(UIDS_ANSWER_SIGN_ALL, GET_LINE, "sign_uid.expire", SET_EXPIRE);
     addEntry(UIDS_ANSWER_SIGN_ALL, GET_LINE, "sign_uid.class", SET_CHECK_LEVEL);
-    addEntry(UIDS_ANSWER_SIGN_ALL, GET_LINE, "trustsig_prompt.trust_value", SET_TRUST_VALUE);
-    addEntry(SET_TRUST_VALUE, GET_LINE, "trustsig_prompt.trust_depth", SET_TRUST_DEPTH);
-    addEntry(SET_TRUST_DEPTH, GET_LINE, "trustsig_prompt.trust_regexp", SET_TRUST_REGEXP);
-    addEntry(SET_TRUST_REGEXP, GET_BOOL, "sign_uid.okay", CONFIRM);
+    addEntry(SET_TRUST_VALUE, GET_LINE, "trustsign_prompt.trust_depth", SET_TRUST_DEPTH);
+    addEntry(SET_TRUST_DEPTH, GET_LINE, "trustsign_prompt.trust_regexp", SET_TRUST_REGEXP);
+    addEntry(SET_TRUST_REGEXP, GET_LINE, "sign_uid.okay", CONFIRM);
     addEntry(SET_CHECK_LEVEL, GET_BOOL, "sign_uid.okay", CONFIRM);
     addEntry(SET_EXPIRE, GET_BOOL, "sign_uid.class", SET_CHECK_LEVEL);
     addEntry(CONFIRM, GET_BOOL, "sign_uid.local_promote_okay", CONFIRM);
-    addEntry(DUPE_OK, GET_BOOL, "sign_uid.okay", CONFIRM);
-    addEntry(DUPE_OK2, GET_BOOL, "sign_uid.okay", CONFIRM);
-    addEntry(DUPE_OK, GET_LINE, "trustsig_prompt.trust_value", SET_TRUST_VALUE);
-    addEntry(DUPE_OK2, GET_LINE, "trustsig_prompt.trust_value", SET_TRUST_VALUE);
     addEntry(CONFIRM, GET_BOOL, "sign_uid.okay", CONFIRM);
-    addEntry(CONFIRM2, GET_BOOL, "sign_uid.okay", CONFIRM);
     addEntry(CONFIRM, GET_LINE, "keyedit.prompt", COMMAND);
-    addEntry(CONFIRM, GET_LINE, "trustsig_prompt.trust_value", SET_TRUST_VALUE);
+    addEntry(CONFIRM, GET_LINE, "trustsign_prompt.trust_value", SET_TRUST_VALUE);
     addEntry(CONFIRM, GET_LINE, "sign_uid.expire", SET_EXPIRE);
     addEntry(CONFIRM, GET_LINE, "sign_uid.class", SET_CHECK_LEVEL);
     addEntry(UIDS_LIST_SEPARATELY_DONE, GET_BOOL, "sign_uid.local_promote_okay", CONFIRM);
     addEntry(UIDS_LIST_SEPARATELY_DONE, GET_LINE, "keyedit.prompt", COMMAND);
-    addEntry(UIDS_LIST_SEPARATELY_DONE, GET_LINE, "trustsig_prompt.trust_value", SET_TRUST_VALUE);
+    addEntry(UIDS_LIST_SEPARATELY_DONE, GET_LINE, "trustsign_prompt.trust_value", SET_TRUST_VALUE);
     addEntry(UIDS_LIST_SEPARATELY_DONE, GET_LINE, "sign_uid.expire", SET_EXPIRE);
     addEntry(UIDS_LIST_SEPARATELY_DONE, GET_LINE, "sign_uid.class", SET_CHECK_LEVEL);
     addEntry(UIDS_LIST_SEPARATELY_DONE, GET_BOOL, "sign_uid.okay", CONFIRM);
-    addEntry(UIDS_LIST_SEPARATELY_DONE, GET_BOOL, "sign_uid.dupe_okay", DUPE_OK);
-    addEntry(DUPE_OK, GET_BOOL, "sign_uid.dupe_okay", DUPE_OK2);
-    addEntry(DUPE_OK2, GET_BOOL, "sign_uid.dupe_okay", DUPE_OK);
     addEntry(CONFIRM, GET_LINE, "keyedit.prompt", QUIT);
-    addEntry(REJECT_SIGN_EXPIRED, GET_LINE, "keyedit.prompt", QUIT);
     addEntry(ERROR, GET_LINE, "keyedit.prompt", QUIT);
     addEntry(QUIT, GET_BOOL, "keyedit.save.okay", SAVE);
 #undef addEntry
@@ -253,22 +219,16 @@ const char *GpgSignKeyEditInteractor::action(Error &err) const
     case SET_EXPIRE:
         return answer(true);
     case SET_TRUST_VALUE:
-        return d->trustSignature.trust == TrustSignatureTrust::Partial ? "1" : "2";
+    // TODO
     case SET_TRUST_DEPTH:
-        return d->trustSignature.depth.c_str();
+    //TODO
     case SET_TRUST_REGEXP:
-        return d->trustSignature.scope.c_str();
+        //TODO
+        return 0;
     case SET_CHECK_LEVEL:
         return check_level_strings[d->checkLevel];
-    case DUPE_OK:
-    case DUPE_OK2:
-        return answer(d->dupeOk);
-    case CONFIRM2:
     case CONFIRM:
         return answer(true);
-    case REJECT_SIGN_EXPIRED:
-        err = Error::fromCode(GPG_ERR_KEY_EXPIRED);
-        return answer(false);
     case QUIT:
         return "quit";
     case SAVE:
@@ -276,24 +236,14 @@ const char *GpgSignKeyEditInteractor::action(Error &err) const
     default:
         if (st >= UIDS_LIST_SEPARATELY && st < UIDS_LIST_SEPARATELY_DONE) {
             std::stringstream ss;
-            auto nextID = d->nextUserID();
-            const char *hash;
-            assert (nextID);
-            if (!d->key.isNull() && (hash = d->key.userID(nextID - 1).uidhash())) {
-                /* Prefer uidhash if it is available as it might happen
-                 * that uidattrs break the ordering of the uids in the
-                 * edit-key interface */
-                ss << "uid " << hash;
-            } else {
-                ss << nextID;
-            }
+            ss << d->nextUserID();
             d->scratch = ss.str();
             return d->scratch.c_str();
         }
     // fall through
     case ERROR:
         err = Error::fromCode(GPG_ERR_GENERAL);
-        return nullptr;
+        return 0;
     }
 }
 
@@ -304,6 +254,9 @@ unsigned int GpgSignKeyEditInteractor::nextState(unsigned int status, const char
     static const Error GENERAL_ERROR = Error::fromCode(GPG_ERR_GENERAL);
     //static const Error INV_TIME_ERROR = Error::fromCode( GPG_ERR_INV_TIME );
     static const TransitionMap table(makeTable());
+    if (needsNoResponse(status)) {
+        return state();
+    }
 
     using namespace GpgSignKeyEditInteractor_Private;
 
@@ -342,10 +295,6 @@ unsigned int GpgSignKeyEditInteractor::nextState(unsigned int status, const char
     err = GENERAL_ERROR;
     return ERROR;
 }
-void GpgSignKeyEditInteractor::setKey(const Key &key)
-{
-    d->key = key;
-}
 
 void GpgSignKeyEditInteractor::setCheckLevel(unsigned int checkLevel)
 {
@@ -367,29 +316,3 @@ void GpgSignKeyEditInteractor::setSigningOptions(int options)
     assert(!d->started);
     d->options = options;
 }
-
-void GpgSignKeyEditInteractor::setDupeOk(bool value)
-{
-    assert(!d->started);
-    d->dupeOk = value;
-}
-
-void GpgSignKeyEditInteractor::setTrustSignatureTrust(GpgME::TrustSignatureTrust trust)
-{
-    assert(!d->started);
-    assert(trust != TrustSignatureTrust::None);
-    d->trustSignature.trust = trust;
-}
-
-void GpgSignKeyEditInteractor::setTrustSignatureDepth(unsigned short depth)
-{
-    assert(!d->started);
-    assert(depth <= 255);
-    d->trustSignature.depth = std::to_string(depth);
-}
-
-void GpgSignKeyEditInteractor::setTrustSignatureScope(const std::string &scope)
-{
-    assert(!d->started);
-    d->trustSignature.scope = scope;
-}
