@@ -1,8 +1,6 @@
 /*
   importresult.cpp - wraps a gpgme import result
   Copyright (C) 2004 Klarälvdalens Datakonsult AB
-  2016 Bundesamt für Sicherheit in der Informationstechnik
-  Software engineering by Intevation GmbH
 
   This file is part of GPGME++.
 
@@ -23,10 +21,6 @@
 */
 
 
-#ifdef HAVE_CONFIG_H
- #include "config.h"
-#endif
-
 #include <importresult.h>
 #include "result_p.h"
 
@@ -35,9 +29,6 @@
 #include <cstring>
 
 #include <string.h>
-#include <strings.h>
-#include <istream>
-#include <iterator>
 
 class GpgME::ImportResult::Private
 {
@@ -49,16 +40,16 @@ public:
         for (gpgme_import_status_t is = r.imports ; is ; is = is->next) {
             gpgme_import_status_t copy = new _gpgme_import_status(*is);
             copy->fpr = strdup(is->fpr);
-            copy->next = nullptr;
+            copy->next = 0;
             imports.push_back(copy);
         }
-        res.imports = nullptr;
+        res.imports = 0;
     }
     ~Private()
     {
         for (std::vector<gpgme_import_status_t>::iterator it = imports.begin() ; it != imports.end() ; ++it) {
             std::free((*it)->fpr);
-            delete *it; *it = nullptr;
+            delete *it; *it = 0;
         }
     }
 
@@ -157,11 +148,6 @@ int GpgME::ImportResult::notImported() const
     return d ? d->res.not_imported : 0 ;
 }
 
-int GpgME::ImportResult::numV3KeysSkipped() const
-{
-    return d ? d->res.skipped_v3_keys : 0 ;
-}
-
 GpgME::Import GpgME::ImportResult::import(unsigned int idx) const
 {
     return Import(d, idx);
@@ -195,7 +181,7 @@ bool GpgME::Import::isNull() const
 
 const char *GpgME::Import::fingerprint() const
 {
-    return isNull() ? nullptr : d->imports[idx]->fpr ;
+    return isNull() ? 0 : d->imports[idx]->fpr ;
 }
 
 GpgME::Error GpgME::Import::error() const
@@ -227,42 +213,3 @@ GpgME::Import::Status GpgME::Import::status() const
     }
     return static_cast<Status>(result);
 }
-
-std::ostream &GpgME::operator<<(std::ostream &os,
-                                const GpgME::ImportResult &result)
-{
-    os << "GpgME::ImportResult(";
-    if (!result.isNull()) {
-        os << "\n considered:          " << result.numConsidered()
-           << "\n without UID:         " << result.numKeysWithoutUserID()
-           << "\n imported:            " << result.numImported()
-           << "\n RSA Imported:        " << result.numRSAImported()
-           << "\n unchanged:           " << result.numUnchanged()
-           << "\n newUserIDs:          " << result.newUserIDs()
-           << "\n newSubkeys:          " << result.newSubkeys()
-           << "\n newSignatures:       " << result.newSignatures()
-           << "\n newRevocations:      " << result.newRevocations()
-           << "\n numSecretKeysConsidered: " << result.numSecretKeysConsidered()
-           << "\n numSecretKeysImported:   " << result.numSecretKeysImported()
-           << "\n numSecretKeysUnchanged:  " << result.numSecretKeysUnchanged()
-           << "\n"
-           << "\n notImported:         " << result.notImported()
-           << "\n numV3KeysSkipped:    " << result.numV3KeysSkipped()
-           << "\n imports:\n";
-        const std::vector<Import> imp = result.imports();
-        std::copy(imp.begin(), imp.end(),
-                  std::ostream_iterator<Import>(os, "\n"));
-    }
-    return os << ')';
-}
-
-std::ostream &GpgME::operator<<(std::ostream &os, const GpgME::Import &imp)
-{
-    os << "GpgME::Import(";
-    if (!imp.isNull()) {
-        os << "\n fpr:       " << (imp.fingerprint() ? imp.fingerprint() : "null")
-           << "\n status:    " << imp.status()
-           << "\n err:       " << imp.error();
-    }
-    return os << ')';
-}
