@@ -20,10 +20,6 @@
   Boston, MA 02110-1301, USA.
 */
 
-#ifdef HAVE_CONFIG_H
- #include "config.h"
-#endif
-
 #include <key.h>
 
 #include "util.h"
@@ -33,7 +29,6 @@
 #include <gpgme.h>
 
 #include <string.h>
-#include <strings.h>
 #include <istream>
 #include <iterator>
 
@@ -165,7 +160,7 @@ Protocol Key::protocol() const
 
 const char *Key::protocolAsString() const
 {
-    return key ? gpgme_get_protocol_name(key->protocol) : nullptr ;
+    return key ? gpgme_get_protocol_name(key->protocol) : 0 ;
 }
 
 bool Key::isRevoked() const
@@ -234,44 +229,28 @@ bool Key::isQualified() const
     return key && key->is_qualified;
 }
 
-bool Key::isDeVs() const
-{
-    if (!key) {
-        return false;
-    }
-    if (!key->subkeys || !key->subkeys->is_de_vs) {
-        return false;
-    }
-    for (gpgme_sub_key_t subkey = key->subkeys ; subkey ; subkey = subkey->next) {
-        if (!subkey->is_de_vs) {
-            return false;
-        }
-    }
-    return true;
-}
-
 const char *Key::issuerSerial() const
 {
-    return key ? key->issuer_serial : nullptr ;
+    return key ? key->issuer_serial : 0 ;
 }
 const char *Key::issuerName() const
 {
-    return key ? key->issuer_name : nullptr ;
+    return key ? key->issuer_name : 0 ;
 }
 const char *Key::chainID() const
 {
-    return key ? key->chain_id : nullptr ;
+    return key ? key->chain_id : 0 ;
 }
 
 const char *Key::keyID() const
 {
-    return key && key->subkeys ? key->subkeys->keyid : nullptr ;
+    return key && key->subkeys ? key->subkeys->keyid : 0 ;
 }
 
 const char *Key::shortKeyID() const
 {
     if (!key || !key->subkeys || !key->subkeys->keyid) {
-        return nullptr;
+        return 0;
     }
     const int len = strlen(key->subkeys->keyid);
     if (len > 8) {
@@ -294,12 +273,11 @@ const char *Key::primaryFingerprint() const
         /* Return the first subkeys fingerprint */
         return key->subkeys->fpr;
     }
-    return nullptr;
 }
 
 unsigned int Key::keyListMode() const
 {
-    return key ? convert_from_gpgme_keylist_mode_t(key->keylist_mode) : 0;
+    return key ? convert_from_gpgme_keylist_mode_t(key->keylist_mode) : 0 ;
 }
 
 const Key &Key::mergeWith(const Key &other)
@@ -347,9 +325,6 @@ const Key &Key::mergeWith(const Key &other)
 
 void Key::update()
 {
-    if (isNull() || !primaryFingerprint()) {
-        return;
-    }
     auto ctx = Context::createForProtocol(protocol());
     if (!ctx) {
         return;
@@ -358,14 +333,9 @@ void Key::update()
                         KeyListMode::Signatures |
                         KeyListMode::SignatureNotations |
                         KeyListMode::Validate |
-                        KeyListMode::WithTofu |
-                        KeyListMode::WithKeygrip);
+                        KeyListMode::WithTofu);
     Error err;
-    auto newKey = ctx->key(primaryFingerprint(), err, true);
-    // Not secret so we get the information from the pubring.
-    if (newKey.isNull()) {
-        newKey = ctx->key(primaryFingerprint(), err, false);
-    }
+    auto newKey = ctx->key(primaryFingerprint(), err, hasSecret());
     delete ctx;
     if (err) {
         return;
@@ -374,27 +344,6 @@ void Key::update()
     return;
 }
 
-// static
-Key Key::locate(const char *mbox)
-{
-    if (!mbox) {
-        return Key();
-    }
-
-    auto ctx = Context::createForProtocol(OpenPGP);
-    if (!ctx) {
-        return Key();
-    }
-
-    ctx->setKeyListMode (Extern | Local);
-
-    Error e = ctx->startKeyListing (mbox);
-    auto ret = ctx->nextKey (e);
-    delete ctx;
-
-    return ret;
-}
-
 //
 //
 // class Subkey
@@ -410,7 +359,7 @@ gpgme_sub_key_t find_subkey(const shared_gpgme_key_t &key, unsigned int idx)
             }
         }
     }
-    return nullptr;
+    return 0;
 }
 
 gpgme_sub_key_t verify_subkey(const shared_gpgme_key_t &key, gpgme_sub_key_t subkey)
@@ -422,10 +371,10 @@ gpgme_sub_key_t verify_subkey(const shared_gpgme_key_t &key, gpgme_sub_key_t sub
             }
         }
     }
-    return nullptr;
+    return 0;
 }
 
-Subkey::Subkey() : key(), subkey(nullptr) {}
+Subkey::Subkey() : key(), subkey(0) {}
 
 Subkey::Subkey(const shared_gpgme_key_t &k, unsigned int idx)
     : key(k), subkey(find_subkey(k, idx))
@@ -446,12 +395,12 @@ Key Subkey::parent() const
 
 const char *Subkey::keyID() const
 {
-    return subkey ? subkey->keyid : nullptr ;
+    return subkey ? subkey->keyid : 0 ;
 }
 
 const char *Subkey::fingerprint() const
 {
-    return subkey ? subkey->fpr : nullptr ;
+    return subkey ? subkey->fpr : 0 ;
 }
 
 Subkey::PubkeyAlgo Subkey::publicKeyAlgorithm() const
@@ -509,11 +458,6 @@ bool Subkey::isQualified() const
     return subkey && subkey->is_qualified;
 }
 
-bool Subkey::isDeVs() const
-{
-    return subkey && subkey->is_de_vs;
-}
-
 bool Subkey::isCardKey() const
 {
     return subkey && subkey->is_cardkey;
@@ -521,12 +465,7 @@ bool Subkey::isCardKey() const
 
 const char *Subkey::cardSerialNumber() const
 {
-    return subkey ? subkey->card_number : nullptr;
-}
-
-const char *Subkey::keyGrip() const
-{
-    return subkey ? subkey->keygrip : nullptr;
+    return subkey ? subkey->card_number : 0 ;
 }
 
 bool Subkey::isSecret() const
@@ -589,7 +528,7 @@ gpgme_user_id_t find_uid(const shared_gpgme_key_t &key, unsigned int idx)
             }
         }
     }
-    return nullptr;
+    return 0;
 }
 
 gpgme_user_id_t verify_uid(const shared_gpgme_key_t &key, gpgme_user_id_t uid)
@@ -601,10 +540,10 @@ gpgme_user_id_t verify_uid(const shared_gpgme_key_t &key, gpgme_user_id_t uid)
             }
         }
     }
-    return nullptr;
+    return 0;
 }
 
-UserID::UserID() : key(), uid(nullptr) {}
+UserID::UserID() : key(), uid(0) {}
 
 UserID::UserID(const shared_gpgme_key_t &k, gpgme_user_id_t u)
     : key(k), uid(verify_uid(k, u))
@@ -656,27 +595,22 @@ std::vector<UserID::Signature> UserID::signatures() const
 
 const char *UserID::id() const
 {
-    return uid ? uid->uid : nullptr ;
+    return uid ? uid->uid : 0 ;
 }
 
 const char *UserID::name() const
 {
-    return uid ? uid->name : nullptr ;
+    return uid ? uid->name : 0 ;
 }
 
 const char *UserID::email() const
 {
-    return uid ? uid->email : nullptr ;
+    return uid ? uid->email : 0 ;
 }
 
 const char *UserID::comment() const
 {
-    return uid ? uid->comment : nullptr ;
-}
-
-const char *UserID::uidhash() const
-{
-    return uid ? uid->uidhash : nullptr ;
+    return uid ? uid->comment : 0 ;
 }
 
 UserID::Validity UserID::validity() const
@@ -729,76 +663,6 @@ TofuInfo UserID::tofuInfo() const
     return TofuInfo(uid->tofu);
 }
 
-static gpgme_key_sig_t find_last_valid_sig_for_keyid (gpgme_user_id_t uid,
-                                                      const char *keyid)
-{
-    if (!keyid) {
-        return nullptr;
-    }
-    gpgme_key_sig_t ret = NULL;
-    for (gpgme_key_sig_t s = uid->signatures ; s ; s = s->next) {
-        if (s->keyid && !strcmp(keyid, s->keyid)) {
-            if (!s->expired && !s->revoked && !s->invalid && !s->status) {
-                if (!ret) {
-                    ret = s;
-                } else if (ret && ret->timestamp <= s->timestamp) {
-                    /* Equals because when the timestamps are the same we prefer
-                       the last in the list */
-                    ret = s;
-                }
-            }
-        }
-    }
-    return ret;
-}
-
-const char *UserID::remark(const Key &remarker, Error &err) const
-{
-    if (!uid || remarker.isNull()) {
-        err = Error::fromCode(GPG_ERR_GENERAL);
-        return nullptr;
-    }
-
-    if (key->protocol != GPGME_PROTOCOL_OpenPGP) {
-        return nullptr;
-    }
-
-    if (!(key->keylist_mode & GPGME_KEYLIST_MODE_SIG_NOTATIONS) ||
-        !(key->keylist_mode & GPGME_KEYLIST_MODE_SIGS)) {
-        err = Error::fromCode(GPG_ERR_NO_DATA);
-        return nullptr;
-    }
-
-    gpgme_key_sig_t s = find_last_valid_sig_for_keyid(uid, remarker.keyID());
-
-    if (!s) {
-        return nullptr;
-    }
-
-    for (gpgme_sig_notation_t n = s->notations; n ; n = n->next) {
-        if (n->name && !strcmp(n->name, "rem@gnupg.org")) {
-            return n->value;
-        }
-    }
-    return nullptr;
-}
-
-std::vector<std::string> UserID::remarks(std::vector<Key> keys, Error &err) const
-{
-    std::vector<std::string> ret;
-
-    for (const auto &key: keys) {
-        const char *rem = remark(key, err);
-        if (err) {
-            return ret;
-        }
-        if (rem) {
-            ret.push_back(rem);
-        }
-    }
-    return ret;
-}
-
 //
 //
 // class Signature
@@ -814,7 +678,7 @@ gpgme_key_sig_t find_signature(gpgme_user_id_t uid, unsigned int idx)
             }
         }
     }
-    return nullptr;
+    return 0;
 }
 
 gpgme_key_sig_t verify_signature(gpgme_user_id_t uid, gpgme_key_sig_t sig)
@@ -826,10 +690,10 @@ gpgme_key_sig_t verify_signature(gpgme_user_id_t uid, gpgme_key_sig_t sig)
             }
         }
     }
-    return nullptr;
+    return 0;
 }
 
-UserID::Signature::Signature() : key(), uid(nullptr), sig(nullptr) {}
+UserID::Signature::Signature() : key(), uid(0), sig(0) {}
 
 UserID::Signature::Signature(const shared_gpgme_key_t &k, gpgme_user_id_t u, unsigned int idx)
     : key(k), uid(verify_uid(k, u)), sig(find_signature(uid, idx))
@@ -850,7 +714,7 @@ UserID UserID::Signature::parent() const
 
 const char *UserID::Signature::signerKeyID() const
 {
-    return sig ? sig->keyid : nullptr ;
+    return sig ? sig->keyid : 0 ;
 }
 
 const char *UserID::Signature::algorithmAsString() const
@@ -900,22 +764,22 @@ bool UserID::Signature::isExportable() const
 
 const char *UserID::Signature::signerUserID() const
 {
-    return sig ? sig->uid : nullptr ;
+    return sig ? sig->uid : 0 ;
 }
 
 const char *UserID::Signature::signerName() const
 {
-    return sig ? sig->name : nullptr ;
+    return sig ? sig->name : 0 ;
 }
 
 const char *UserID::Signature::signerEmail() const
 {
-    return sig ? sig->email : nullptr ;
+    return sig ? sig->email : 0 ;
 }
 
 const char *UserID::Signature::signerComment() const
 {
-    return sig ? sig->comment : nullptr ;
+    return sig ? sig->comment : 0 ;
 }
 
 unsigned int UserID::Signature::certClass() const
@@ -998,134 +862,14 @@ std::vector<Notation> UserID::Signature::notations() const
 const char *UserID::Signature::policyURL() const
 {
     if (!sig) {
-        return nullptr;
+        return 0;
     }
     for (gpgme_sig_notation_t nota = sig->notations ; nota ; nota = nota->next) {
         if (!nota->name) {
             return nota->value;
         }
     }
-    return nullptr;
-}
-
-std::string UserID::addrSpecFromString(const char *userid)
-{
-    if (!userid) {
-        return std::string();
-    }
-    char *normalized = gpgme_addrspec_from_uid (userid);
-    if (normalized) {
-        std::string ret(normalized);
-        gpgme_free(normalized);
-        return ret;
-    }
-    return std::string();
-}
-
-std::string UserID::addrSpec() const
-{
-    if (!uid || !uid->address) {
-        return std::string();
-    }
-
-    return uid->address;
-}
-
-Error UserID::revoke()
-{
-    if (isNull()) {
-        return Error::fromCode(GPG_ERR_GENERAL);
-    }
-    auto ctx = Context::createForProtocol(parent().protocol());
-    if (!ctx) {
-        return Error::fromCode(GPG_ERR_INV_ENGINE);
-    }
-    Error ret = ctx->revUid(key, id());
-    delete ctx;
-    return ret;
-}
-
-static Key::Origin gpgme_origin_to_pp_origin (const unsigned int origin)
-{
-    switch (origin) {
-        case GPGME_KEYORG_KS:
-            return Key::OriginKS;
-        case GPGME_KEYORG_DANE:
-            return Key::OriginDane;
-        case GPGME_KEYORG_WKD:
-            return Key::OriginWKD;
-        case GPGME_KEYORG_URL:
-            return Key::OriginURL;
-        case GPGME_KEYORG_FILE:
-            return Key::OriginFile;
-        case GPGME_KEYORG_SELF:
-            return Key::OriginSelf;
-        case GPGME_KEYORG_OTHER:
-            return Key::OriginOther;
-        case GPGME_KEYORG_UNKNOWN:
-        default:
-            return Key::OriginUnknown;
-    }
-}
-
-Key::Origin UserID::origin() const
-{
-    if (isNull()) {
-        return Key::OriginUnknown;
-    }
-    return gpgme_origin_to_pp_origin(uid->origin);
-}
-
-time_t UserID::lastUpdate() const
-{
-    return static_cast<time_t>(uid ? uid->last_update : 0);
-}
-
-Error Key::addUid(const char *uid)
-{
-    if (isNull()) {
-        return Error::fromCode(GPG_ERR_GENERAL);
-    }
-    auto ctx = Context::createForProtocol(protocol());
-    if (!ctx) {
-        return Error::fromCode(GPG_ERR_INV_ENGINE);
-    }
-    Error ret = ctx->addUid(key, uid);
-    delete ctx;
-    return ret;
-}
-
-Key::Origin Key::origin() const
-{
-    if (isNull()) {
-        return OriginUnknown;
-    }
-    return gpgme_origin_to_pp_origin(key->origin);
-}
-
-time_t Key::lastUpdate() const
-{
-    return static_cast<time_t>(key ? key->last_update : 0);
-}
-
-bool Key::isBad() const
-{
-    return isNull() || isRevoked() || isExpired() || isDisabled() || isInvalid();
-}
-
-bool Subkey::isBad() const
-{
-    return isNull() || isRevoked() || isExpired() || isDisabled() || isInvalid();
-}
-
-bool UserID::isBad() const
-{
-    return isNull() || isRevoked() || isInvalid();
-}
-
-bool UserID::Signature::isBad() const
-{
-    return isNull() || isExpired() || isInvalid();
+    return 0;
 }
 
 std::ostream &operator<<(std::ostream &os, const UserID &uid)
@@ -1134,14 +878,11 @@ std::ostream &operator<<(std::ostream &os, const UserID &uid)
     if (!uid.isNull()) {
         os << "\n name:      " << protect(uid.name())
            << "\n email:     " << protect(uid.email())
-           << "\n mbox:      " << uid.addrSpec()
            << "\n comment:   " << protect(uid.comment())
            << "\n validity:  " << uid.validityAsString()
            << "\n revoked:   " << uid.isRevoked()
            << "\n invalid:   " << uid.isInvalid()
            << "\n numsigs:   " << uid.numSignatures()
-           << "\n origin:    " << uid.origin()
-           << "\n updated:   " << uid.lastUpdate()
            << "\n tofuinfo:\n" << uid.tofuInfo();
     }
     return os << ')';
@@ -1160,8 +901,6 @@ std::ostream &operator<<(std::ostream &os, const Key &key)
            << "\n canEncrypt: " << key.canEncrypt()
            << "\n canCertify: " << key.canCertify()
            << "\n canAuth:    " << key.canAuthenticate()
-           << "\n origin:     " << key.origin()
-           << "\n updated:    " << key.lastUpdate()
            << "\n uids:\n";
         const std::vector<UserID> uids = key.userIDs();
         std::copy(uids.begin(), uids.end(),
