@@ -112,7 +112,6 @@ public:
     bool canCertify() const;
     bool canAuthenticate() const;
     bool isQualified() const;
-    bool isDeVs() const;
 
     bool hasSecret() const;
     GPGMEPP_DEPRECATED bool isSecret() const
@@ -153,52 +152,6 @@ public:
      * how long the keylisting takes.*/
     void update();
 
-    /**
-     * @brief Add a user id to this key.
-     *
-     * Needs gnupg 2.1.13 and the key needs to be updated
-     * afterwards to see the new uid.
-     *
-     * @param uid should be fully formated and UTF-8 encoded.
-     *
-     * @returns a possible error.
-     **/
-    Error addUid(const char *uid);
-
-    /**
-     * @brief try to locate the best pgp key for a given mailbox.
-     *
-     * Boils down to gpg --locate-key <mbox>
-     * This may take some time if remote sources are also
-     * used.
-     *
-     * @param mbox should be a mail address does not need to be normalized.
-     *
-     * @returns The best key for a mailbox or a null key.
-     */
-    static Key locate(const char *mbox);
-
-    /* @enum Origin
-     * @brief The Origin of the key. */
-    enum Origin : unsigned int {
-        OriginUnknown = 0,
-        OriginKS = 1,
-        OriginDane = 3,
-        OriginWKD = 4,
-        OriginURL = 5,
-        OriginFile = 6,
-        OriginSelf = 7,
-        OriginOther = 31,
-    };
-    /*! Get the origin of the key.
-     *
-     * @returns the Origin. */
-    Origin origin() const;
-
-    /*! Get the last update time.
-     *
-     * @returns the last update time. */
-    time_t lastUpdate() const;
 private:
     gpgme_key_t impl() const
     {
@@ -255,7 +208,6 @@ public:
     bool canCertify() const;
     bool canAuthenticate() const;
     bool isQualified() const;
-    bool isDeVs() const;
     bool isCardKey() const;
 
     bool isSecret() const;
@@ -307,8 +259,6 @@ public:
 
     const char *cardSerialNumber() const;
 
-    const char *keyGrip() const;
-
 private:
     shared_gpgme_key_t key;
     gpgme_sub_key_t subkey;
@@ -370,38 +320,6 @@ public:
      * @returns The TOFU stats or a null TofuInfo.
      */
     GpgME::TofuInfo tofuInfo() const;
-
-    /*! Wrapper around gpgme_addrspec_from_uid.
-     *
-     * The input string should match the format of
-     * a user id string.
-     *
-     * @returns a normalized mail address if found
-     * or an empty string. */
-    static std::string addrSpecFromString(const char *uid);
-
-    /*! Wrapper around gpgme_addrspec_from_uid.
-     *
-     * @returns a normalized mail address for this userid
-     * or an empty string. */
-    std::string addrSpec() const;
-
-    /*! Revoke the user id.
-     *
-     * Key needs update afterwards.
-     *
-     * @returns an error on error.*/
-    Error revoke();
-
-    /*! Get the origin of the key.
-     *
-     * @returns the Origin. */
-    Key::Origin origin() const;
-
-    /*! Get the last update time.
-     *
-     * @returns the last update time. */
-    time_t lastUpdate() const;
 private:
     shared_gpgme_key_t key;
     gpgme_user_id_t uid;
