@@ -14,15 +14,15 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
+
 """Demonstrate the use of the Assuan protocol engine"""
 
 from __future__ import absolute_import, print_function, unicode_literals
-
-import gpg
-
 del absolute_import, print_function, unicode_literals
 
-with gpg.Context(protocol=gpg.constants.protocol.ASSUAN) as c:
+import pyme
+
+with pyme.Context(protocol=pyme.constants.PROTOCOL_ASSUAN) as c:
     # Invoke the pinentry to get a confirmation.
     err = c.assuan_transact(['GET_CONFIRMATION', 'Hello there'])
     print("You chose {}.".format("cancel" if err else "ok"))
