@@ -14,21 +14,21 @@
 # General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, see <https://www.gnu.org/licenses/>.
+# along with this program; if not, see <http://www.gnu.org/licenses/>.
+
 """A test applicaton for the CMS protocol."""
 
 from __future__ import absolute_import, print_function, unicode_literals
+del absolute_import, print_function, unicode_literals
 
 import sys
-import gpg
-
-del absolute_import, print_function, unicode_literals
+import pyme
 
 if len(sys.argv) != 2:
     sys.exit("fingerprint or unique key ID for gpgme_get_key()")
 
-with gpg.Context(protocol=gpg.constants.protocol.CMS) as c:
-    key = c.get_key(sys.argv[1])
+with pyme.Context(protocol=pyme.constants.PROTOCOL_CMS) as c:
+    key = c.get_key(sys.argv[1], False)
 
     print("got key: ", key.subkeys[0].fpr)
     for uid in key.uids:
