@@ -18,17 +18,14 @@
 # License along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import absolute_import, print_function, unicode_literals
+del absolute_import, print_function, unicode_literals
 
 import io
 import os
 import tempfile
-import gpg
-import support
-_ = support  # to appease pyflakes.
+from pyme import core
 
-del absolute_import, print_function, unicode_literals
-
-data = gpg.Data('Hello world!')
+data = core.Data('Hello world!')
 assert data.read() == b'Hello world!'
 assert data.read() == b''
 
@@ -36,35 +33,29 @@ data.seek(0, os.SEEK_SET)
 assert data.read() == b'Hello world!'
 assert data.read() == b''
 
-data = gpg.Data(b'Hello world!')
+data = core.Data(b'Hello world!')
 assert data.read() == b'Hello world!'
 
-data = gpg.Data(b'Hello world!', copy=False)
+data = core.Data(b'Hello world!', copy=False)
 assert data.read() == b'Hello world!'
 
-data = gpg.Data()
+data = core.Data()
 data.write('Hello world!')
 data.seek(0, os.SEEK_SET)
 assert data.read() == b'Hello world!'
 
-data = gpg.Data()
+data = core.Data()
 data.write(b'Hello world!')
 data.seek(0, os.SEEK_SET)
 assert data.read() == b'Hello world!'
 
-data = gpg.Data()
-data.write(b'Hello world!')
-# We expect the second argument to default to SEEK_SET
-data.seek(0)
-assert data.read() == b'Hello world!'
-
 binjunk = bytes(range(256))
-data = gpg.Data()
+data = core.Data()
 data.write(binjunk)
 data.seek(0, os.SEEK_SET)
 assert data.read() == binjunk
 
-data = gpg.Data()
+data = core.Data()
 data.set_file_name("foobar")
 assert data.get_file_name() == "foobar"
 
@@ -75,28 +66,27 @@ with tempfile.NamedTemporaryFile() as tmp:
     tmp.seek(0)
 
     # Open using name.
-    data = gpg.Data(file=tmp.name)
+    data = core.Data(file=tmp.name)
     assert data.read() == binjunk
 
     # Open using name, without copying.
     if False:
         # delayed reads are not yet supported
-        data = gpg.Data(file=tmp.name, copy=False)
+        data = core.Data(file=tmp.name, copy=False)
         assert data.read() == binjunk
 
     # Open using stream.
     tmp.seek(0)
-    data = gpg.Data(file=tmp)
+    data = core.Data(file=tmp)
     assert data.read() == binjunk
 
     # Open using stream, offset, and length.
-    data = gpg.Data(file=tmp, offset=0, length=42)
+    data = core.Data(file=tmp, offset=0, length=42)
     assert data.read() == binjunk[:42]
 
     # Open using name, offset, and length.
-    data = gpg.Data(file=tmp.name, offset=23, length=42)
-    assert data.read() == binjunk[23:23 + 42]
-
+    data = core.Data(file=tmp.name, offset=23, length=42)
+    assert data.read() == binjunk[23:23+42]
 
 # Test callbacks.
 class DataObject(object):
@@ -120,10 +110,9 @@ class DataObject(object):
         assert not self.released
         self.released = True
 
-
 do = DataObject()
 cookie = object()
-data = gpg.Data(cbs=(do.read, do.write, do.seek, do.release, cookie))
+data = core.Data(cbs=(do.read, do.write, do.seek, do.release, cookie))
 data.write('Hello world!')
 data.seek(0, os.SEEK_SET)
 assert data.read() == b'Hello world!'
@@ -132,7 +121,7 @@ assert do.released
 
 # Again, without the cookie.
 do = DataObject()
-data = gpg.Data(cbs=(do.read, do.write, do.seek, do.release))
+data = core.Data(cbs=(do.read, do.write, do.seek, do.release))
 data.write('Hello world!')
 data.seek(0, os.SEEK_SET)
 assert data.read() == b'Hello world!'
