@@ -18,16 +18,17 @@
 # License along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import absolute_import, print_function, unicode_literals
-
-import gpg
-import support
-
 del absolute_import, print_function, unicode_literals
 
-c = gpg.Context()
+import pyme
+from pyme import core, constants
+import support
 
-source = gpg.Data(file=support.make_filename("cipher-1.asc"))
-sink = gpg.Data()
+support.init_gpgme(constants.PROTOCOL_OpenPGP)
+c = core.Context()
+
+source = core.Data(file=support.make_filename("cipher-1.asc"))
+sink = core.Data()
 
 c.op_decrypt(source, sink)
 result = c.op_decrypt_result()
@@ -37,7 +38,7 @@ assert not result.unsupported_algorithm, \
 support.print_data(sink)
 
 # Idiomatic interface.
-with gpg.Context() as c:
+with pyme.Context() as c:
     plaintext, _, _ = c.decrypt(open(support.make_filename("cipher-1.asc")))
     assert len(plaintext) > 0
     assert plaintext.find(b'Wenn Sie dies lesen k') >= 0, \
