@@ -15,27 +15,27 @@
 # Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, see <https://www.gnu.org/licenses/>.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import absolute_import, print_function, unicode_literals
-
-import os
-import gpg
-import support
-_ = support  # to appease pyflakes.
-
 del absolute_import, print_function, unicode_literals
 
-for passphrase in ("abc", b"abc"):
-    c = gpg.Context()
-    c.set_armor(True)
-    c.set_pinentry_mode(gpg.constants.PINENTRY_MODE_LOOPBACK)
+import os
+import pyme
+from pyme import core, constants
+import support
 
-    source = gpg.Data("Hallo Leute\n")
-    cipher = gpg.Data()
+support.init_gpgme(constants.PROTOCOL_OpenPGP)
+
+for passphrase in ("abc", b"abc"):
+    c = core.Context()
+    c.set_armor(True)
+    c.set_pinentry_mode(constants.PINENTRY_MODE_LOOPBACK)
+
+    source = core.Data("Hallo Leute\n")
+    cipher = core.Data()
 
     passphrase_cb_called = 0
-
     def passphrase_cb(hint, desc, prev_bad, hook=None):
         global passphrase_cb_called
         passphrase_cb_called += 1
@@ -44,28 +44,20 @@ for passphrase in ("abc", b"abc"):
     c.set_passphrase_cb(passphrase_cb, None)
 
     c.op_encrypt([], 0, source, cipher)
-    # gpg 2.2.21 has a bug in that for a new passphrase the callback
-    # is called twice.  This is fixed in 2.2.22 but a patch was also
-    # distributed so that we allow both.
-    if support.is_gpg_version((2,2,21)):
-        print("Enabling GnuPG 2.2.21 bug 4991 test workaround.")
-        assert passphrase_cb_called == 1 or passphrase_cb_called == 2, \
-            "Callback called {} times".format(passphrase_cb_called)
-    else:
-        assert passphrase_cb_called == 1, \
-            "Callback called {} times".format(passphrase_cb_called)
+    assert passphrase_cb_called == 1, \
+        "Callback called {} times".format(passphrase_cb_called)
     support.print_data(cipher)
 
-    c = gpg.Context()
+    c = core.Context()
     c.set_armor(True)
-    c.set_pinentry_mode(gpg.constants.PINENTRY_MODE_LOOPBACK)
+    c.set_pinentry_mode(constants.PINENTRY_MODE_LOOPBACK)
     c.set_passphrase_cb(passphrase_cb, None)
-    plain = gpg.Data()
+    plain = core.Data()
     cipher.seek(0, os.SEEK_SET)
 
     c.op_decrypt(cipher, plain)
     # Seems like the passphrase is cached.
-    # assert passphrase_cb_called == 2, \
+    #assert passphrase_cb_called == 2, \
     #    "Callback called {} times".format(passphrase_cb_called)
     support.print_data(plain)
 
@@ -76,16 +68,16 @@ for passphrase in ("abc", b"abc"):
 
 # Idiomatic interface.
 for passphrase in ("abc", b"abc"):
-    with gpg.Context(armor=True) as c:
+    with pyme.Context(armor=True) as c:
         # Check that the passphrase callback is not altered.
         def f(*args):
             assert False
-
         c.set_passphrase_cb(f)
 
         message = "Hallo Leute\n".encode()
-        ciphertext, _, _ = c.encrypt(
-            message, passphrase=passphrase, sign=False)
+        ciphertext, _, _ = c.encrypt(message,
+                                     passphrase=passphrase,
+                                     sign=False)
         assert ciphertext.find(b'BEGIN PGP MESSAGE') > 0, 'Marker not found'
 
         plaintext, _, _ = c.decrypt(ciphertext, passphrase=passphrase)
