@@ -21,23 +21,21 @@ from __future__ import absolute_import, print_function, unicode_literals
 del absolute_import, print_function, unicode_literals
 
 import os
-import gpg
+from pyme import core, constants
 import support
-_ = support # to appease pyflakes.
 
 expected_notations = {
-    "laughing@me": ("Just Squeeze Me", gpg.constants.sig.notation.HUMAN_READABLE),
+    "laughing@me": ("Just Squeeze Me", constants.SIG_NOTATION_HUMAN_READABLE),
     "preferred-email-encoding@pgp.com": ("pgpmime",
-                                         gpg.constants.sig.notation.HUMAN_READABLE
-                                         | gpg.constants.sig.notation.CRITICAL),
+                                         constants.SIG_NOTATION_HUMAN_READABLE
+                                         | constants.SIG_NOTATION_CRITICAL),
     None: ("http://www.gnu.org/policy/", 0),
 }
 
 # GnuPG prior to 2.1.13 did not report the critical flag correctly.
-with gpg.Context() as c:
+with core.Context() as c:
     version = c.engine_info.version
     have_correct_sig_data = not (version.startswith("1.")
-                                 or version.startswith("2.0.")
                                  or version == "2.1.1"
                                  or (version.startswith("2.1.1")
                                      and version[5] < '3'))
@@ -56,24 +54,26 @@ def check_result(result):
         assert r.value == value, \
             "Expected {!r}, got {!r}".format(value, r.value)
         assert r.human_readable \
-            == bool(flags & gpg.constants.sig.notation.HUMAN_READABLE)
+            == bool(flags&constants.SIG_NOTATION_HUMAN_READABLE)
         assert r.critical \
-            == (bool(flags & gpg.constants.sig.notation.CRITICAL)
+            == (bool(flags&constants.SIG_NOTATION_CRITICAL)
                 if have_correct_sig_data else False)
 
     assert len(expected_notations) == 0
 
-source = gpg.Data("Hallo Leute\n")
-signed = gpg.Data()
+support.init_gpgme(constants.PROTOCOL_OpenPGP)
 
-c = gpg.Context()
+source = core.Data("Hallo Leute\n")
+signed = core.Data()
+
+c = core.Context()
 for name, (value, flags) in expected_notations.items():
     c.sig_notation_add(name, value, flags)
 
-c.op_sign(source, signed, gpg.constants.sig.mode.NORMAL)
+c.op_sign(source, signed, constants.SIG_MODE_NORMAL)
 
 signed.seek(0, os.SEEK_SET)
-sink = gpg.Data()
+sink = core.Data()
 c.op_verify(signed, None, sink)
 result = c.op_verify_result()
 check_result(result)
