@@ -15,21 +15,19 @@
 # Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, see <https://www.gnu.org/licenses/>.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import absolute_import, print_function, unicode_literals
-
-import os
-import gpg
-import support
-
 del absolute_import, print_function, unicode_literals
 
+import os
+import pyme
+from pyme import core, constants
+import support
 
 def fail(msg):
     raise RuntimeError(msg)
 
-
 def check_result(r, typ):
     if r.invalid_signers:
         fail("Invalid signer found: {}".format(r.invalid_signers.fpr))
@@ -41,71 +39,74 @@ def check_result(r, typ):
     if signature.type != typ:
         fail("Wrong type of signature created")
 
-    if signature.pubkey_algo != gpg.constants.pk.DSA:
+    if signature.pubkey_algo != constants.PK_DSA:
         fail("Wrong pubkey algorithm reported: {}".format(
             signature.pubkey_algo))
 
-    if signature.hash_algo != gpg.constants.md.SHA1:
-        fail("Wrong hash algorithm reported: {}".format(signature.hash_algo))
+    if signature.hash_algo != constants.MD_SHA1:
+        fail("Wrong hash algorithm reported: {}".format(
+            signature.hash_algo))
 
     if signature.sig_class != 1:
-        fail("Wrong signature class reported: {}".format(signature.sig_class))
+        fail("Wrong signature class reported: {}".format(
+            signature.sig_class))
 
     if signature.fpr != "A0FF4590BB6122EDEF6E3C542D727CC768697734":
         fail("Wrong fingerprint reported: {}".format(signature.fpr))
 
 
-c = gpg.Context()
+support.init_gpgme(constants.PROTOCOL_OpenPGP)
+c = core.Context()
 c.set_textmode(True)
 c.set_armor(True)
 
-source = gpg.Data("Hallo Leute\n")
-sink = gpg.Data()
+source = core.Data("Hallo Leute\n")
+sink = core.Data()
 
-c.op_sign(source, sink, gpg.constants.sig.mode.NORMAL)
+c.op_sign(source, sink, constants.SIG_MODE_NORMAL)
 
 result = c.op_sign_result()
-check_result(result, gpg.constants.sig.mode.NORMAL)
+check_result(result, constants.SIG_MODE_NORMAL)
 support.print_data(sink)
 
 # Now a detached signature.
 source.seek(0, os.SEEK_SET)
-sink = gpg.Data()
+sink = core.Data()
 
-c.op_sign(source, sink, gpg.constants.sig.mode.DETACH)
+c.op_sign(source, sink, constants.SIG_MODE_DETACH)
 
 result = c.op_sign_result()
-check_result(result, gpg.constants.sig.mode.DETACH)
+check_result(result, constants.SIG_MODE_DETACH)
 support.print_data(sink)
 
 # And finally a cleartext signature.  */
 source.seek(0, os.SEEK_SET)
-sink = gpg.Data()
+sink = core.Data()
 
-c.op_sign(source, sink, gpg.constants.sig.mode.CLEAR)
+c.op_sign(source, sink, constants.SIG_MODE_CLEAR)
 
 result = c.op_sign_result()
-check_result(result, gpg.constants.sig.mode.CLEAR)
+check_result(result, constants.SIG_MODE_CLEAR)
 support.print_data(sink)
 
 # Idiomatic interface.
-with gpg.Context(armor=True, textmode=True) as c:
+with pyme.Context(armor=True, textmode=True) as c:
     message = "Hallo Leute\n".encode()
     signed, _ = c.sign(message)
     assert len(signed) > 0
     assert signed.find(b'BEGIN PGP MESSAGE') > 0, 'Message not found'
 
-    signed, _ = c.sign(message, mode=gpg.constants.sig.mode.DETACH)
+    signed, _ = c.sign(message, mode=pyme.constants.SIG_MODE_DETACH)
     assert len(signed) > 0
     assert signed.find(b'BEGIN PGP SIGNATURE') > 0, 'Signature not found'
 
-    signed, _ = c.sign(message, mode=gpg.constants.sig.mode.CLEAR)
+    signed, _ = c.sign(message, mode=pyme.constants.SIG_MODE_CLEAR)
     assert len(signed) > 0
     assert signed.find(b'BEGIN PGP SIGNED MESSAGE') > 0, 'Message not found'
     assert signed.find(message) > 0, 'Message content not found'
     assert signed.find(b'BEGIN PGP SIGNATURE') > 0, 'Signature not found'
 
-with gpg.Context() as c:
+with pyme.Context() as c:
     message = "Hallo Leute\n".encode()
 
     c.signers = [c.get_key(support.sign_only, True)]
@@ -114,7 +115,7 @@ with gpg.Context() as c:
     c.signers = [c.get_key(support.encrypt_only, True)]
     try:
         c.sign(message)
-    except gpg.errors.InvalidSigners as e:
+    except pyme.errors.InvalidSigners as e:
         assert len(e.signers) == 1
         assert support.encrypt_only.endswith(e.signers[0].fpr)
     else:
