@@ -15,24 +15,21 @@
 # Public License for more details.
 #
 # You should have received a copy of the GNU Lesser General Public
-# License along with this program; if not, see <https://www.gnu.org/licenses/>.
+# License along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import absolute_import, print_function, unicode_literals
-
-import gpg
-import support
-_ = support  # to appease pyflakes.
-
 del absolute_import, print_function, unicode_literals
 
-c = gpg.Context()
+from pyme import core, constants
+import support
 
+support.init_gpgme(constants.PROTOCOL_OpenPGP)
+c = core.Context()
 
 def dump_item(item):
-    print("l={} k={} t={} o={} v={} u={}".format(item.level, item.keyid,
-                                                 item.type, item.owner_trust,
-                                                 item.validity, item.name))
-
+    print("l={} k={} t={} o={} v={} u={}".format(
+        item.level, item.keyid, item.type, item.owner_trust,
+        item.validity, item.name))
 
 c.op_trustlist_start("alice", 0)
 while True:
