@@ -18,17 +18,16 @@
 # License along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 from __future__ import absolute_import, print_function, unicode_literals
+del absolute_import, print_function, unicode_literals
 
 import sys
 import os
-import gpg
+import pyme
+from pyme import core, constants, errors
 import support
-_ = support  # to appease pyflakes.
-
-del absolute_import, print_function, unicode_literals
 
 test_text1 = b"Just GNU it!\n"
-test_text1f = b"Just GNU it?\n"
+test_text1f= b"Just GNU it?\n"
 test_sig1 = b"""-----BEGIN PGP SIGNATURE-----
 
 iN0EABECAJ0FAjoS+i9FFIAAAAAAAwA5YmFyw7bDpMO8w58gZGFzIHdhcmVuIFVt
@@ -61,7 +60,6 @@ UqVooWlGXHwNw/xg/fVzt9VNbtjtJ/fhUqYo0/LyCGEA
 -----END PGP MESSAGE-----
 """
 
-
 def check_result(result, summary, validity, fpr, status, notation):
     assert len(result.signatures) == 1, "Unexpected number of signatures"
     sig = result.signatures[0]
@@ -69,7 +67,7 @@ def check_result(result, summary, validity, fpr, status, notation):
         "Unexpected signature summary: {}, want: {}".format(sig.summary,
                                                             summary)
     assert sig.fpr == fpr
-    assert gpg.errors.GPGMEError(sig.status).getcode() == status
+    assert errors.GPGMEError(sig.status).getcode() == status
 
     if notation:
         expected_notations = {
@@ -78,16 +76,14 @@ def check_result(result, summary, validity, fpr, status, notation):
                     if sys.version_info[0] < 3 else
                     b"\xc3\xb6\xc3\xa4\xc3\xbc\xc3\x9f".decode() +
                     " das waren Umlaute und jetzt ein prozent%-Zeichen"),
-            "foobar.1":
-            "this is a notation data with 2 lines",
-            None:
-            "http://www.gu.org/policy/",
+            "foobar.1":  "this is a notation data with 2 lines",
+            None: "http://www.gu.org/policy/",
         }
         assert len(sig.notations) == len(expected_notations)
 
         for r in sig.notations:
-            assert 'name_len' not in dir(r)
-            assert 'value_len' not in dir(r)
+            assert not 'name_len' in dir(r)
+            assert not 'value_len' in dir(r)
             assert r.name in expected_notations
             assert r.value == expected_notations[r.name], \
                 "Expected {!r}, got {!r}".format(expected_notations[r.name],
@@ -100,85 +96,87 @@ def check_result(result, summary, validity, fpr, status, notation):
     assert sig.validity == validity, \
         "Unexpected signature validity: {}, want: {}".format(
             sig.validity, validity)
-    assert gpg.errors.GPGMEError(
-        sig.validity_reason).getcode() == gpg.errors.NO_ERROR
+    assert errors.GPGMEError(sig.validity_reason).getcode() == errors.NO_ERROR
 
 
-c = gpg.Context()
+support.init_gpgme(constants.PROTOCOL_OpenPGP)
+c = core.Context()
 c.set_armor(True)
 
 # Checking a valid message.
-text = gpg.Data(test_text1)
-sig = gpg.Data(test_sig1)
+text = core.Data(test_text1)
+sig = core.Data(test_sig1)
 c.op_verify(sig, text, None)
 result = c.op_verify_result()
-check_result(result, gpg.constants.sigsum.VALID | gpg.constants.sigsum.GREEN,
-             gpg.constants.validity.FULL,
-             "A0FF4590BB6122EDEF6E3C542D727CC768697734", gpg.errors.NO_ERROR,
-             True)
+check_result(result, constants.SIGSUM_VALID | constants.SIGSUM_GREEN,
+             constants.VALIDITY_FULL,
+             "A0FF4590BB6122EDEF6E3C542D727CC768697734",
+             errors.NO_ERROR, True)
+
 
 # Checking a manipulated message.
-text = gpg.Data(test_text1f)
+text = core.Data(test_text1f)
 sig.seek(0, os.SEEK_SET)
 c.op_verify(sig, text, None)
 result = c.op_verify_result()
-check_result(result, gpg.constants.sigsum.RED, gpg.constants.validity.UNKNOWN,
-             "2D727CC768697734", gpg.errors.BAD_SIGNATURE, False)
+check_result(result, constants.SIGSUM_RED, constants.VALIDITY_UNKNOWN,
+             "2D727CC768697734", errors.BAD_SIGNATURE, False)
 
 # Checking a normal signature.
-text = gpg.Data()
-sig = gpg.Data(test_sig2)
+text = core.Data()
+sig = core.Data(test_sig2)
 c.op_verify(sig, None, text)
 result = c.op_verify_result()
-check_result(result, gpg.constants.sigsum.VALID | gpg.constants.sigsum.GREEN,
-             gpg.constants.validity.FULL,
-             "A0FF4590BB6122EDEF6E3C542D727CC768697734", gpg.errors.NO_ERROR,
-             False)
+check_result(result, constants.SIGSUM_VALID | constants.SIGSUM_GREEN,
+             constants.VALIDITY_FULL,
+             "A0FF4590BB6122EDEF6E3C542D727CC768697734",
+             errors.NO_ERROR, False)
 
 # Checking an invalid message.
-text = gpg.Data()
-sig = gpg.Data(double_plaintext_sig)
+text = core.Data()
+sig = core.Data(double_plaintext_sig)
 try:
     c.op_verify(sig, None, text)
 except Exception as e:
-    assert type(e) == gpg.errors.GPGMEError
-    assert e.getcode() == gpg.errors.BAD_DATA
+    assert type(e) == errors.GPGMEError
+    assert e.getcode() == errors.BAD_DATA
 else:
     assert False, "Expected an error but got none."
 
+
 # Idiomatic interface.
-with gpg.Context(armor=True) as c:
+with pyme.Context(armor=True) as c:
     # Checking a valid message.
     _, result = c.verify(test_text1, test_sig1)
-    check_result(
-        result, gpg.constants.sigsum.VALID | gpg.constants.sigsum.GREEN,
-        gpg.constants.validity.FULL,
-        "A0FF4590BB6122EDEF6E3C542D727CC768697734", gpg.errors.NO_ERROR, True)
+    check_result(result, constants.SIGSUM_VALID | constants.SIGSUM_GREEN,
+                 constants.VALIDITY_FULL,
+                 "A0FF4590BB6122EDEF6E3C542D727CC768697734",
+                 errors.NO_ERROR, True)
 
     # Checking a manipulated message.
     try:
         c.verify(test_text1f, test_sig1)
-    except gpg.errors.BadSignatures as e:
-        check_result(e.result, gpg.constants.sigsum.RED,
-                     gpg.constants.validity.UNKNOWN, "2D727CC768697734",
-                     gpg.errors.BAD_SIGNATURE, False)
+    except errors.BadSignatures as e:
+        check_result(e.result, constants.SIGSUM_RED,
+                     constants.VALIDITY_UNKNOWN,
+                     "2D727CC768697734", errors.BAD_SIGNATURE, False)
     else:
         assert False, "Expected an error but got none."
 
     # Checking a normal signature.
-    sig = gpg.Data(test_sig2)
+    sig = core.Data(test_sig2)
     data, result = c.verify(test_sig2)
-    check_result(
-        result, gpg.constants.sigsum.VALID | gpg.constants.sigsum.GREEN,
-        gpg.constants.validity.FULL,
-        "A0FF4590BB6122EDEF6E3C542D727CC768697734", gpg.errors.NO_ERROR, False)
+    check_result(result, constants.SIGSUM_VALID | constants.SIGSUM_GREEN,
+                 constants.VALIDITY_FULL,
+                 "A0FF4590BB6122EDEF6E3C542D727CC768697734",
+                 errors.NO_ERROR, False)
     assert data == test_text1
 
     # Checking an invalid message.
     try:
         c.verify(double_plaintext_sig)
-    except gpg.errors.GPGMEError as e:
-        assert e.getcode() == gpg.errors.BAD_DATA
+    except errors.GPGMEError as e:
+        assert e.getcode() == errors.BAD_DATA
     else:
         assert False, "Expected an error but got none."
 
@@ -190,7 +188,7 @@ with gpg.Context(armor=True) as c:
 
     try:
         c.verify(test_text1, test_sig1, verify=[alpha, bob])
-    except gpg.errors.MissingSignatures as e:
+    except errors.MissingSignatures as e:
         assert len(e.missing) == 1
         assert e.missing[0] == bob
     else:
