@@ -28,10 +28,6 @@
     your version.
 */
 
-#ifdef HAVE_CONFIG_H
- #include "config.h"
-#endif
-
 #include "defaultkeygenerationjob.h"
 #include "protocol.h"
 #include "keygenerationjob.h"
@@ -91,11 +87,6 @@ void DefaultKeyGenerationJob::slotCancel()
 
 GpgME::Error DefaultKeyGenerationJob::start(const QString &email, const QString &name)
 {
-    const QString namePart = name.isEmpty() ? QString() :
-                                QStringLiteral("name-real:     %1\n").arg(name);
-    const QString mailPart = email.isEmpty() ? QString() :
-                                QStringLiteral("name-email:    %1\n").arg(email);
-
     const QString args = QStringLiteral("<GnupgKeyParms format=\"internal\">\n"
                                         "%ask-passphrase\n"
                                         "key-type:      RSA\n"
@@ -104,17 +95,17 @@ GpgME::Error DefaultKeyGenerationJob::start(const QString &email, const QString
                                         "subkey-type:   RSA\n"
                                         "subkey-length: 2048\n"
                                         "subkey-usage:  encrypt\n"
-                                        "%1"
-                                        "%2"
-                                        "</GnupgKeyParms>").arg(mailPart, namePart);
+                                        "name-email:    %1\n"
+                                        "name-real:     %2\n"
+                                        "</GnupgKeyParms>").arg(email, name);
 
     d->job = openpgp()->keyGenerationJob();
     d->job->installEventFilter(this);
-    connect(d->job.data(), &KeyGenerationJob::result,
+    connect(d->job, &KeyGenerationJob::result,
             this, &DefaultKeyGenerationJob::result);
-    connect(d->job.data(), &KeyGenerationJob::done,
+    connect(d->job, &KeyGenerationJob::done,
             this, &DefaultKeyGenerationJob::done);
-    connect(d->job.data(), &KeyGenerationJob::done,
+    connect(d->job, &KeyGenerationJob::done,
             this, &QObject::deleteLater);
     return d->job->start(args);
 }
