@@ -1,19 +1,19 @@
 /* engine.h - GPGME engine interface.
    Copyright (C) 2000 Werner Koch (dd9jn)
    Copyright (C) 2001, 2002, 2003, 2004, 2010 g10 Code GmbH
- 
+
    This file is part of GPGME.
- 
+
    GPGME is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; either version 2.1 of
    the License, or (at your option) any later version.
-   
+
    GPGME is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
-   
+
    You should have received a copy of the GNU Lesser General Public
    License along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -23,7 +23,12 @@
 #define ENGINE_H
 
 #include "gpgme.h"
- 
+
+/* Flags used by the EXTRAFLAGS arg of _gpgme_engine_op_genkey.  */
+#define GENKEY_EXTRAFLAG_ARMOR   1
+#define GENKEY_EXTRAFLAG_REVOKE  2
+
+
 struct engine;
 typedef struct engine *engine_t;
 
@@ -38,6 +43,8 @@ typedef gpgme_error_t (*engine_command_handler_t) (void *priv,
 typedef gpgme_error_t (*engine_assuan_result_cb_t) (void *priv,
                                                     gpgme_error_t result);
 
+/* Helper for gpgme_set_global_flag.  */
+int _gpgme_set_engine_minimal_version (const char *value);
 
 /* Get a deep copy of the engine info and return it in INFO.  */
 gpgme_error_t _gpgme_engine_info_copy (gpgme_engine_info_t *r_info);
@@ -62,6 +69,8 @@ gpgme_error_t _gpgme_engine_set_locale (engine_t engine, int category,
 gpgme_error_t _gpgme_engine_set_protocol (engine_t engine,
 					  gpgme_protocol_t protocol);
 void _gpgme_engine_release (engine_t engine);
+void _gpgme_engine_set_status_cb (engine_t engine,
+                                  gpgme_status_cb_t cb, void *cb_value);
 void _gpgme_engine_set_status_handler (engine_t engine,
 				       engine_status_handler_t fnc,
 				       void *fnc_value);
@@ -104,21 +113,36 @@ gpgme_error_t _gpgme_engine_op_export_ext (engine_t engine,
 					   gpgme_data_t keydata,
 					   int use_armor);
 gpgme_error_t _gpgme_engine_op_genkey (engine_t engine,
+                                       const char *userid, const char *algo,
+                                       unsigned long reserved,
+                                       unsigned long expires,
+                                       gpgme_key_t key, unsigned int flags,
 				       gpgme_data_t help_data,
-				       int use_armor, gpgme_data_t pubkey,
+				       unsigned int extraflags,
+                                       gpgme_data_t pubkey,
 				       gpgme_data_t seckey);
+gpgme_error_t _gpgme_engine_op_keysign (engine_t engine,
+                                        gpgme_key_t key, const char *userid,
+                                        unsigned long expires,
+                                        unsigned int flags,
+                                        gpgme_ctx_t ctx);
+gpgme_error_t _gpgme_engine_op_tofu_policy (engine_t engine,
+                                            gpgme_key_t key,
+                                            gpgme_tofu_policy_t policy);
 gpgme_error_t _gpgme_engine_op_import (engine_t engine,
 				       gpgme_data_t keydata,
                                        gpgme_key_t *keyarray);
 gpgme_error_t _gpgme_engine_op_keylist (engine_t engine,
 					const char *pattern,
 					int secret_only,
-					gpgme_keylist_mode_t mode);
+					gpgme_keylist_mode_t mode,
+					int engine_flags);
 gpgme_error_t _gpgme_engine_op_keylist_ext (engine_t engine,
 					    const char *pattern[],
 					    int secret_only,
 					    int reserved,
-					    gpgme_keylist_mode_t mode);
+					    gpgme_keylist_mode_t mode,
+					    int engine_flags);
 gpgme_error_t _gpgme_engine_op_sign (engine_t engine, gpgme_data_t in,
 				     gpgme_data_t out, gpgme_sig_mode_t mode,
 				     int use_armor, int use_textmode,
@@ -133,8 +157,8 @@ gpgme_error_t _gpgme_engine_op_verify (engine_t engine, gpgme_data_t sig,
 gpgme_error_t _gpgme_engine_op_getauditlog (engine_t engine,
                                             gpgme_data_t output,
                                             unsigned int flags);
-gpgme_error_t _gpgme_engine_op_assuan_transact 
-                (engine_t engine, 
+gpgme_error_t _gpgme_engine_op_assuan_transact
+                (engine_t engine,
                  const char *command,
                  gpgme_assuan_data_cb_t data_cb,
                  void *data_cb_value,
@@ -160,5 +184,18 @@ gpgme_error_t _gpgme_engine_cancel_op (engine_t engine);
 gpgme_error_t _gpgme_engine_op_passwd (engine_t engine, gpgme_key_t key,
                                        unsigned int flags);
 
+gpgme_error_t _gpgme_engine_set_pinentry_mode (engine_t engine,
+                                               gpgme_pinentry_mode_t mode);
+
+gpgme_error_t _gpgme_engine_op_spawn (engine_t engine,
+                                      const char *file, const char *argv[],
+                                      gpgme_data_t datain,
+                                      gpgme_data_t dataout,
+                                      gpgme_data_t dataerr,
+                                      unsigned int flags);
+
+/* The available engine option flags.  */
+#define GPGME_ENGINE_FLAG_OFFLINE        1
+
 
 #endif /* ENGINE_H */
