@@ -14,8 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #if HAVE_CONFIG_H
@@ -172,7 +171,7 @@ keysign_start (gpgme_ctx_t ctx, int synchronous,
   if (ctx->passphrase_cb)
     {
       err = _gpgme_engine_set_command_handler
-        (ctx->engine, _gpgme_passphrase_command_handler, ctx);
+        (ctx->engine, _gpgme_passphrase_command_handler, ctx, NULL);
       if (err)
         return err;
     }
@@ -189,7 +188,7 @@ gpgme_op_keysign_start (gpgme_ctx_t ctx, gpgme_key_t key, const char *userid,
 {
   gpgme_error_t err;
 
-  TRACE_BEG  (DEBUG_CTX, "gpgme_op_keysign_start", ctx,
+  TRACE_BEG3 (DEBUG_CTX, "gpgme_op_keysign_start", ctx,
 	      "key=%p, uid='%s' flags=0x%x", key, userid, flags);
 
   if (!ctx)
@@ -206,7 +205,7 @@ gpgme_op_keysign (gpgme_ctx_t ctx, gpgme_key_t key, const char *userid,
 {
   gpgme_error_t err;
 
-  TRACE_BEG  (DEBUG_CTX, "gpgme_op_keysign", ctx,
+  TRACE_BEG3 (DEBUG_CTX, "gpgme_op_keysign", ctx,
 	      "key=%p, uid='%s' flags=0x%x", key, userid, flags);
 
   if (!ctx)
