@@ -23,6 +23,9 @@
 #endif
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_STDINT_H
+# include <stdint.h>
+#endif
 #include <string.h>
 #include <assert.h>
 #include <errno.h>
@@ -44,6 +47,12 @@
 #include <ctype.h>
 #include <sys/resource.h>
 
+#if __linux__
+# include <sys/types.h>
+# include <dirent.h>
+#endif /*__linux__ */
+
+
 #include "util.h"
 #include "priv-io.h"
 #include "sema.h"
@@ -273,20 +282,54 @@ _gpgme_io_set_nonblocking (int fd)
 static long int
 get_max_fds (void)
 {
-  char *source = NULL;
+  const char *source = NULL;
   long int fds = -1;
   int rc;
 
-#ifdef RLIMIT_NOFILE
+  /* Under Linux we can figure out the highest used file descriptor by
+   * reading /proc/self/fd.  This is in the common cases much fast than
+   * for example doing 4096 close calls where almost all of them will
+   * fail.  */
+#ifdef __linux__
   {
-    struct rlimit rl;
-    rc = getrlimit (RLIMIT_NOFILE, &rl);
-    if (rc == 0)
+    DIR *dir = NULL;
+    struct dirent *dir_entry;
+    const char *s;
+    int x;
+
+    dir = opendir ("/proc/self/fd");
+    if (dir)
       {
-	source = "RLIMIT_NOFILE";
-	fds = rl.rlim_max;
+        while ((dir_entry = readdir (dir)))
+          {
+            s = dir_entry->d_name;
+            if ( *s < '0' || *s > '9')
+              continue;
+            x = atoi (s);
+            if (x > fds)
+              fds = x;
+          }
+        closedir (dir);
       }
-  }
+    if (fds != -1)
+      {
+        fds++;
+        source = "/proc";
+      }
+    }
+#endif /* __linux__ */
+
+#ifdef RLIMIT_NOFILE
+  if (fds == -1)
+    {
+      struct rlimit rl;
+      rc = getrlimit (RLIMIT_NOFILE, &rl);
+      if (rc == 0)
+        {
+          source = "RLIMIT_NOFILE";
+          fds = rl.rlim_max;
+        }
+    }
 #endif
 #ifdef RLIMIT_OFILE
   if (fds == -1)
@@ -331,6 +374,16 @@ get_max_fds (void)
       fds = 1024;
     }
 
+  /* AIX returns INT32_MAX instead of a proper value.  We assume that
+   * this is always an error and use a more reasonable limit.  */
+#ifdef INT32_MAX
+  if (fds == INT32_MAX)
+    {
+      source = "aix-fix";
+      fds = 1024;
+    }
+#endif
+
   TRACE2 (DEBUG_SYSIO, "gpgme:max_fds", 0, "max fds=%i (%s)", fds, source);
   return fds;
 }
@@ -377,8 +430,6 @@ _gpgme_io_spawn (const char *path, char *const argv[], unsigned int flags,
   int status;
   int signo;
 
-  (void)flags;
-
   TRACE_BEG1 (DEBUG_SYSIO, "_gpgme_io_spawn", path,
 	      "path=%s", path);
   i = 0;
@@ -407,6 +458,7 @@ _gpgme_io_spawn (const char *path, char *const argv[], unsigned int flags,
 
 	  /* Child.  */
 	  int seen_stdin = 0;
+	  int seen_stdout = 0;
 	  int seen_stderr = 0;
 
 	  if (atfork)
@@ -435,6 +487,8 @@ _gpgme_io_spawn (const char *path, char *const argv[], unsigned int flags,
 
 	      if (child_fd == 0)
 		seen_stdin = 1;
+	      else if (child_fd == 1)
+		seen_stdout = 1;
 	      else if (child_fd == 2)
 		seen_stderr = 1;
 
@@ -456,56 +510,38 @@ _gpgme_io_spawn (const char *path, char *const argv[], unsigned int flags,
 	      close (fd_list[i].fd);
 	    }
 
-	  if (! seen_stdin || ! seen_stderr)
+	  if (! seen_stdin || ! seen_stdout || !seen_stderr)
 	    {
 	      fd = open ("/dev/null", O_RDWR);
 	      if (fd == -1)
 		{
-#if 0
-		  /* FIXME: The debug file descriptor is not dup'ed
-		     anyway, so we can't see this.  */
-		  TRACE_LOG1 ("can't open `/dev/null': %s\n",
-			      strerror (errno));
-#endif
+		  /* The debug file descriptor is not dup'ed, so we
+		     can't do a trace output.  */
 		  _exit (8);
 		}
-	      /* Make sure that the process has a connected stdin.  */
+	      /* Make sure that the process has connected stdin.  */
 	      if (! seen_stdin && fd != 0)
 		{
 		  if (dup2 (fd, 0) == -1)
-		    {
-#if 0
-		  /* FIXME: The debug file descriptor is not dup'ed
-		     anyway, so we can't see this.  */
-		      TRACE_LOG1 ("dup2(/dev/null, 0) failed: %s\n",
-				  strerror (errno));
-#endif
-		      _exit (8);
-		    }
+                    _exit (8);
 		}
+	      if (! seen_stdout && fd != 1)
+                {
+                  if (dup2 (fd, 1) == -1)
+                    _exit (8);
+                }
 	      if (! seen_stderr && fd != 2)
-		if (dup2 (fd, 2) == -1)
-		  {
-#if 0
-		    /* FIXME: The debug file descriptor is not dup'ed
-		       anyway, so we can't see this.  */
-		    TRACE_LOG1 ("dup2(dev/null, 2) failed: %s\n",
-				strerror (errno));
-#endif
-		    _exit (8);
-		  }
-	      if (fd != 0 && fd != 2)
+                {
+                  if (dup2 (fd, 2) == -1)
+                    _exit (8);
+                }
+	      if (fd != 0 && fd != 1 && fd != 2)
 		close (fd);
 	    }
 
 	  execv (path, (char *const *) argv);
 	  /* Hmm: in that case we could write a special status code to the
 	     status-pipe.  */
-#if 0
-	  /* FIXME: The debug file descriptor is not dup'ed anyway, so
-	     we can't see this.  */
-	  TRACE_LOG1 ("exec of `%s' failed\n", path);
-#endif
 	  _exit (8);
 	  /* End child.  */
 	}
@@ -568,6 +604,12 @@ _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
 	continue;
       if (fds[i].for_read)
 	{
+          if (fds[i].fd >= FD_SETSIZE)
+            {
+              TRACE_END (dbg_help, " -BAD- ]");
+              gpg_err_set_errno (EBADF);
+              return TRACE_SYSRES (-1);
+            }
 	  assert (!FD_ISSET (fds[i].fd, &readfds));
 	  FD_SET (fds[i].fd, &readfds);
 	  if (fds[i].fd > max_fd)
@@ -577,6 +619,12 @@ _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
         }
       else if (fds[i].for_write)
 	{
+          if (fds[i].fd >= FD_SETSIZE)
+            {
+              TRACE_END (dbg_help, " -BAD- ]");
+              gpg_err_set_errno (EBADF);
+              return TRACE_SYSRES (-1);
+            }
 	  assert (!FD_ISSET (fds[i].fd, &writefds));
 	  FD_SET (fds[i].fd, &writefds);
 	  if (fds[i].fd > max_fd)
