@@ -14,8 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /* We need to include config.h so that we know whether we are building
@@ -38,19 +37,6 @@
 static int verbose;
 
 
-static char *
-xstrdup (const char *string)
-{
-  char *p = strdup (string);
-  if (!p)
-    {
-      fprintf (stderr, "strdup failed\n");
-      exit (2);
-    }
-  return p;
-}
-
-
 static gpg_error_t
 status_cb (void *opaque, const char *keyword, const char *value)
 {
@@ -94,19 +80,15 @@ show_usage (int ex)
 {
   fputs ("usage: " PGM " [options] FILE\n\n"
          "Options:\n"
-         "  --verbose          run in verbose mode\n"
-         "  --status           print status lines from the backend\n"
-         "  --progress         print progress info\n"
-         "  --openpgp          use the OpenPGP protocol (default)\n"
-         "  --cms              use the CMS protocol\n"
-         "  --uiserver         use the UI server\n"
-         "  --loopback         use a loopback pinentry\n"
-         "  --key NAME         encrypt to key NAME\n"
-         "  --keystring NAMES  encrypt to ';' delimited NAMES\n"
-         "  --throw-keyids     use this option\n"
-         "  --no-symkey-cache  disable the use of that cache\n"
-         "  --wrap             assume input is valid OpenPGP message\n"
-         "  --symmetric        encrypt symmetric (OpenPGP only)\n"
+         "  --verbose        run in verbose mode\n"
+         "  --status         print status lines from the backend\n"
+         "  --progress       print progress info\n"
+         "  --openpgp        use the OpenPGP protocol (default)\n"
+         "  --cms            use the CMS protocol\n"
+         "  --uiserver       use the UI server\n"
+         "  --loopback       use a loopback pinentry\n"
+         "  --key NAME       encrypt to key NAME\n"
+         "  --symmetric      encrypt symmetric (OpenPGP only)\n"
          , stderr);
   exit (ex);
 }
@@ -118,6 +100,7 @@ main (int argc, char **argv)
   int last_argc = -1;
   gpgme_error_t err;
   gpgme_ctx_t ctx;
+  const char *key_string = NULL;
   gpgme_protocol_t protocol = GPGME_PROTOCOL_OpenPGP;
   gpgme_data_t in, out;
   gpgme_encrypt_result_t result;
@@ -127,11 +110,9 @@ main (int argc, char **argv)
   char *keyargs[10];
   gpgme_key_t keys[10+1];
   int keycount = 0;
-  char *keystring = NULL;
   int i;
   gpgme_encrypt_flags_t flags = GPGME_ENCRYPT_ALWAYS_TRUST;
   gpgme_off_t offset;
-  int no_symkey_cache = 0;
 
   if (argc)
     { argc--; argv++; }
@@ -189,27 +170,6 @@ main (int argc, char **argv)
           keyargs[keycount++] = *argv;
           argc--; argv++;
         }
-      else if (!strcmp (*argv, "--keystring"))
-        {
-          argc--; argv++;
-          if (!argc)
-            show_usage (1);
-          keystring = xstrdup (*argv);
-          for (i=0; keystring[i]; i++)
-            if (keystring[i] == ';')
-              keystring[i] = '\n';
-          argc--; argv++;
-        }
-      else if (!strcmp (*argv, "--throw-keyids"))
-        {
-          flags |= GPGME_ENCRYPT_THROW_KEYIDS;
-          argc--; argv++;
-        }
-      else if (!strcmp (*argv, "--wrap"))
-        {
-          flags |= GPGME_ENCRYPT_WRAP;
-          argc--; argv++;
-        }
       else if (!strcmp (*argv, "--loopback"))
         {
           use_loopback = 1;
@@ -220,11 +180,6 @@ main (int argc, char **argv)
           flags |= GPGME_ENCRYPT_SYMMETRIC;
           argc--; argv++;
         }
-      else if (!strcmp (*argv, "--no-symkey-cache"))
-        {
-          no_symkey_cache = 1;
-          argc--; argv++;
-        }
       else if (!strncmp (*argv, "--", 2))
         show_usage (1);
 
@@ -233,6 +188,15 @@ main (int argc, char **argv)
   if (argc != 1)
     show_usage (1);
 
+  if (key_string && protocol == GPGME_PROTOCOL_UISERVER)
+    {
+      fprintf (stderr, PGM ": ignoring --key in UI-server mode\n");
+      key_string = NULL;
+    }
+
+  if (!key_string)
+    key_string = "test";
+
   init_gpgme (protocol);
 
   err = gpgme_new (&ctx);
@@ -251,16 +215,6 @@ main (int argc, char **argv)
       gpgme_set_pinentry_mode (ctx, GPGME_PINENTRY_MODE_LOOPBACK);
       gpgme_set_passphrase_cb (ctx, passphrase_cb, NULL);
     }
-  if (no_symkey_cache)
-    {
-      err = gpgme_set_ctx_flag (ctx, "no-symkey-cache", "1");
-      if (err)
-        {
-          fprintf (stderr, PGM ": error setting no-symkey-cache:  %s\n",
-                   gpgme_strerror (err));
-          exit (1);
-        }
-    }
 
   for (i=0; i < keycount; i++)
     {
@@ -315,8 +269,7 @@ main (int argc, char **argv)
   err = gpgme_data_new (&out);
   fail_if_err (err);
 
-  err = gpgme_op_encrypt_ext (ctx, keycount ? keys : NULL, keystring,
-                              flags, in, out);
+  err = gpgme_op_encrypt (ctx, keycount ? keys : NULL, flags, in, out);
   result = gpgme_op_encrypt_result (ctx);
   if (result)
     print_result (result);
@@ -336,6 +289,5 @@ main (int argc, char **argv)
   for (i=0; i < keycount; i++)
     gpgme_key_unref (keys[i]);
   gpgme_release (ctx);
-  free (keystring);
   return 0;
 }
